@@ -1,12 +1,12 @@
 #!/usr/bin/perl
 ###########################################################################
 # Makefile for ABI Compliance Checker
-# Install/remove the tool for GNU/Linux and FreeBSD
+# Install/remove the tool for GNU/Linux, FreeBSD and Mac OS X
 #
 # Copyright (C) 2009-2010 The Linux Foundation
 # Copyright (C) 2009-2011 Institute for System Programming, RAS
 # Copyright (C) 2011-2012 Nokia Corporation and/or its subsidiary(-ies)
-# Copyright (C) 2011-2012 ROSA Laboratory
+# Copyright (C) 2011-2013 ROSA Laboratory
 #
 # Written by Andrey Ponomarenko
 #
@@ -26,6 +26,7 @@
 use Getopt::Long;
 Getopt::Long::Configure ("posix_default", "no_ignore_case");
 use File::Path qw(mkpath rmtree);
+use File::Spec qw(catfile file_name_is_absolute);
 use File::Copy qw(copy);
 use File::Basename qw(dirname);
 use Cwd qw(abs_path);
@@ -44,9 +45,9 @@ DESCRIPTION:
   Install $TOOL_SNAME command and private modules.
 
 USAGE:
-  sudo perl $0 -install -prefix=/usr
-  sudo perl $0 -update -prefix=/usr
-  sudo perl $0 -remove -prefix=/usr
+  sudo perl $0 -install -prefix /usr
+  sudo perl $0 -update -prefix /usr
+  sudo perl $0 -remove -prefix /usr
 
 OPTIONS:
   -h|-help
@@ -72,7 +73,8 @@ EXTRA OPTIONS:
       supported.
 \n";
 
-if(not @ARGV) {
+if(not @ARGV)
+{
     print $HELP_MSG;
     exit(0);
 }
@@ -90,17 +92,13 @@ GetOptions(
 
 sub scenario()
 {
-    if($Help) {
+    if($Help)
+    {
         print $HELP_MSG;
         exit(0);
     }
-    if($Config{"osname"}!~/linux|freebsd|openbsd|netbsd/)
-    { # MS Windows, Mac OS X
-        print STDERR "The tool is ready-to-use without the need to install.\n";
-        print STDERR "This Makefile is for GNU/Linux and BSD systems.\n";
-        exit(1);
-    }
-    if(not $Install and not $Update and not $Remove) {
+    if(not $Install and not $Update and not $Remove)
+    {
         print STDERR "ERROR: command is not selected (-install, -update or -remove)\n";
         exit(1);
     }
@@ -109,7 +107,9 @@ sub scenario()
     }
     if(not $PREFIX)
     { # default prefix
-        $PREFIX = "/usr/local";
+        if($Config{"osname"}!~/win/i) {
+            $PREFIX = "/usr/local";
+        }
     }
     if(my $Var = $ENV{"DESTDIR"})
     {
@@ -121,27 +121,32 @@ sub scenario()
         if($DESTDIR ne "/") {
             $DESTDIR=~s/[\/]+\Z//g;
         }
-        if($DESTDIR!~/\A\//) {
+        if(not isAbs($DESTDIR))
+        {
             print STDERR "ERROR: destdir is not absolute path\n";
             exit(1);
         }
-        if(not -d $DESTDIR) {
+        if(not -d $DESTDIR)
+        {
             print STDERR "ERROR: you should create destdir directory first\n";
             exit(1);
         }
         $PREFIX = $DESTDIR.$PREFIX;
-        if(not -d $PREFIX) {
+        if(not -d $PREFIX)
+        {
             print STDERR "ERROR: you should create installation directory first (destdir + prefix):\n  mkdir -p $PREFIX\n";
             exit(1);
         }
     }
     else
     {
-        if($PREFIX!~/\A\//) {
+        if(not isAbs($PREFIX))
+        {
             print STDERR "ERROR: prefix is not absolute path\n";
             exit(1);
         }
-        if(not -d $PREFIX) {
+        if(not -d $PREFIX)
+        {
             print STDERR "ERROR: you should create prefix directory first\n";
             exit(1);
         }
@@ -150,30 +155,37 @@ sub scenario()
     print "INSTALL PREFIX: $PREFIX\n";
     
     # paths
-    my $EXE_PATH = "$PREFIX/bin";
-    my $MODULES_PATH = "$PREFIX/share/$TOOL_SNAME";
-    my $REL_PATH = "../share/$TOOL_SNAME";
+    my $EXE_PATH = catFile($PREFIX, "bin");
+    my $MODULES_PATH = catFile($PREFIX, "share", $TOOL_SNAME);
+    my $REL_PATH = catFile("..", "share", $TOOL_SNAME);
+    my $TOOL_PATH = catFile($EXE_PATH, $TOOL_SNAME);
     
-    if(not -w $PREFIX) {
+    if(not -w $PREFIX)
+    {
         print STDERR "ERROR: you should be root\n";
         exit(1);
     }
     if($Remove or $Update)
     {
-        # remove executable
-        print "-- Removing $EXE_PATH/$TOOL_SNAME\n";
-        unlink($EXE_PATH."/".$TOOL_SNAME);
+        if(-e $EXE_PATH."/".$TOOL_SNAME)
+        { # remove executable
+            print "-- Removing $TOOL_PATH\n";
+            unlink($EXE_PATH."/".$TOOL_SNAME);
+        }
         
-        # remove modules
-        print "-- Removing $MODULES_PATH\n";
-        rmtree($MODULES_PATH);
+        if(-d $MODULES_PATH)
+        { # remove modules
+            print "-- Removing $MODULES_PATH\n";
+            rmtree($MODULES_PATH);
+        }
     }
     if($Install or $Update)
     {
         if(-e $EXE_PATH."/".$TOOL_SNAME or -e $MODULES_PATH)
         { # check installed
-            if(not $Remove) {
-                print STDERR "ERROR: you should remove old version first (`sudo perl $0 -remove --prefix=$PREFIX`)\n";
+            if(not $Remove)
+            {
+                print STDERR "ERROR: you should remove old version first (`perl $0 -remove --prefix=$PREFIX`)\n";
                 exit(1);
             }
         }
@@ -181,18 +193,22 @@ sub scenario()
         # configure
         my $Content = readFile($ARCHIVE_DIR."/".$TOOL_SNAME.".pl");
         if($DESTDIR) { # relative path
-            $Content=~s/ACC_MODULES_INSTALL_PATH/$REL_PATH/;
+            $Content=~s/MODULES_INSTALL_PATH/$REL_PATH/;
         }
         else { # absolute path
-            $Content=~s/ACC_MODULES_INSTALL_PATH/$MODULES_PATH/;
+            $Content=~s/MODULES_INSTALL_PATH/$MODULES_PATH/;
         }
         
         # copy executable
-        print "-- Installing $EXE_PATH/$TOOL_SNAME\n";
+        print "-- Installing $TOOL_PATH\n";
         mkpath($EXE_PATH);
         writeFile($EXE_PATH."/".$TOOL_SNAME, $Content);
         chmod(0775, $EXE_PATH."/".$TOOL_SNAME);
         
+        if($Config{"osname"}=~/win/i) {
+            writeFile($EXE_PATH."/".$TOOL_SNAME.".cmd", "\@perl \"$TOOL_PATH\" \%*");
+        }
+        
         # copy modules
         if(-d $ARCHIVE_DIR."/modules")
         {
@@ -202,13 +218,32 @@ sub scenario()
         }
         
         # check PATH
-        if($ENV{"PATH"}!~/(\A|:)\Q$EXE_PATH\E(\Z|:)/) {
-            print "WARNING: your PATH variable doesn't include \'$EXE_PATH\'\n";
+        my $Warn = "WARNING: your PATH variable doesn't include \'$EXE_PATH\'\n";
+        
+        if($Config{"osname"}=~/win/i)
+        {
+            if($ENV{"PATH"}!~/(\A|[:;])\Q$EXE_PATH\E[\/\\]?(\Z|[:;])/i) { 
+                print $Warn;
+            }
+        }
+        else
+        {
+            if($ENV{"PATH"}!~/(\A|[:;])\Q$EXE_PATH\E[\/\\]?(\Z|[:;])/) { 
+                print $Warn;
+            }
         }
     }
     exit(0);
 }
 
+sub catFile(@) {
+    return File::Spec->catfile(@_);
+}
+
+sub isAbs($) {
+    return File::Spec->file_name_is_absolute($_[0]);
+}
+
 sub copyDir($$)
 {
     my ($From, $To) = @_;
@@ -220,7 +255,7 @@ sub copyDir($$)
     foreach my $Path (sort keys(%Files))
     {
         if($Config{"osname"}!~/win/ and $Path=~/Targets\/(windows|symbian)/) {
-                next;
+            next;
         }
         my $Inst = $Path;
         $Inst=~s/\A\Q$ARCHIVE_DIR\E/$To/;
